from datetime import datetime
from bash import bash
import sqlalchemy as db
from sqlalchemy.orm import sessionmaker
import pandas as pd
from pprint import pprint

import imaplib
import smtplib
import email

class AirflowClient :
    def __init__(self,usr='airflow',pwd='airflow',host='10.30.225.172',port='5432',dbname='airflow-dev', airflow_bin='/home/airflow/venv/bin/airflow'):
        self.engine = db.create_engine(f'postgresql+psycopg2://{usr}:{pwd}@{host}:{port}/{dbname}')
        self.connection = self.engine.connect()
        self.metadata = db.MetaData()
        Session = sessionmaker()
        Session.configure(bind=self.engine)

        self.session = Session()


        self.airflow = airflow_bin

    def dag_status(self, dag_id, prc_dt=datetime.now().strftime('%Y%m%d')):
        dag = db.Table('dag_run',self.metadata, autoload=True, autoload_with=self.engine)
        print(f'{prc_dt} 00:00:00')
        dt_start = datetime.strptime(f'{prc_dt} 00:00:00','%Y%m%d %H:%M:%S').strftime('%Y-%m-%d %H:%M:%S')
        dt_end = datetime.strptime(f'{prc_dt} 23:59:59','%Y%m%d %H:%M:%S').strftime('%Y-%m-%d %H:%M:%S')

        print('start',dt_start, 'end', dt_end)
        res = self.session.query(dag).filter(dag.columns.dag_id == dag_id)\
                .filter(dag.columns.execution_date >= dt_start )\
                .filter(dag.columns.execution_date <= dt_end )\
                .all() #.filter(dag.columns.execution_date == datetime.strptime(prc_dt , '%Y%m%d').strftime('%Y-%m-%d'))
        
        df = pd.DataFrame(res)
        return df['state'].values[0]

    def run_dag(self,dag_id, prc_dt):
        exec_date = datetime.strptime(prc_dt, '%Y%m%d').strftime('%Y-%m-%d')
        run_cmd = f'''   
            {self.airflow} trigger_dag -e {exec_date} {dag_id}

            if [[ $? -ne 0 ]]; then
                {self.airflow} clear -s {exec_date} -e {exec_date} -c {dag_id}
            fi
        '''
        bash(run_cmd)

class OMTClient:
    
    def __init__(self,login, password):
        self.login = login
        self.password = password

    def is_success(self, omt_signature, prc_dt):
        print("login mail")
        mail=imaplib.IMAP4_SSL("imap.gmail.com")
        mail.login(self.login,self.password)
        mail.select(mailbox='INBOX')
        # SUCCESS 20200423
        #
        typ,data=mail.search(None,f'(FROM "bigdata.support@ovo.id" SUBJECT "[SUCCESS] {omt_signature} - RAW, process Date: {prc_dt}")')
        print("get response")
        # pprint(data)

        for num in data[0].split():
            typ, data = mail.fetch(num, '(RFC822)')
            for response_part in data :
                
                if isinstance(response_part,tuple):
                    
                    msg=email.message_from_string(response_part[1].decode('utf-8'))
                    # print(msg)
                    print ("subj:", msg['subject'])
                    print ("from:", msg['from'])
                    print ("body:")

                    return True
                    
        mail.close()
        mail.logout()


def main():
    # airflow = AirflowClient()
    # airflow.run_dag(dag_id='ovo_smp_db_daily_pten_regist',prc_dt='20200420')
    # res = airflow.dag_status(dag_id='ovo_smp_db_daily_pten_regist', prc_dt='20200415')
    # print(res)

    omt = OMTClient()
    stat = omt.is_success(omt_signature='OVO_SMP_MLM_MERCHANT',prc_dt='20200302')
    print(f"IS SUCCESS {stat}")

if __name__ == "__main__":
    main()


'''
RAW TEST

implementation of simple test case (with less typing)

**how to run?**
```/usr/bin/spark2-submit $(pwd)/raw_test.py```
'''

# import unittest
# import findspark
# findspark.init('/opt/cloudera/parcels/SPARK2/lib/spark2/')

from pyspark.sql import SparkSession
from pyspark.sql.functions import lit,expr, explode, explode_outer, map_values, from_unixtime, unix_timestamp, to_timestamp
from pyspark.sql.types import *
import sys, os
from datetime import datetime,timedelta

import unittest
import logging
from pyspark.sql import SparkSession


class OVOTestKit(unittest.TestCase):
 
    @classmethod
    def suppress_py4j_logging(cls):
        logger = logging.getLogger('py4j')
        logger.setLevel(logging.WARN)

    @classmethod
    def create_testing_pyspark_session(cls):
        return (SparkSession.builder.appName(__name__)\
            .config('hive.exec.dynamic.partition.mode','nonstrict')\
            .enableHiveSupport()\
            .getOrCreate()) 
 
    @classmethod
    def setUpClass(cls):
        cls.suppress_py4j_logging()
        cls.spark = cls.create_testing_pyspark_session()

    @classmethod
    def tearDownClass(cls):
        cls.spark.stop()

    def _normalize(self, s):
        return str(s).lower().replace(' ','')

    def field_check(self, table_name, fields, types):
        '''
            table_name : complete table name including db name
            fields : fields from modeller
            types : types from modeller
        '''
        dm_colname = list(map(self._normalize, fields))
        dm_coltype = list(map(self._normalize, types))

        t_struct = self.spark.sql(f"describe {table_name}").toPandas()
        t_struct = t_struct[['col_name','data_type']].to_dict(orient='list')
        t_struct = [ x for x in zip(t_struct['col_name'], t_struct['data_type'])]

        d_struct = [ x for x in zip(list(map(self._normalize, dm_colname)), list(map(self._normalize, dm_coltype)) )]

        self.assertEqual(t_struct, d_struct)
    
    def duplicate_check(self, table_name, columns = ['id']):

        grouper = ','.join(columns)

        Q = f'select {grouper} , count(1) N from {table_name} group by {grouper} having count(1) > 1'
        df = self.spark.sql(Q).toPandas()
        row, col = df.shape

        self.assertEqual(row, 0 , '{} duplicated row found')

    def random_check(self, table_name):
        NotImplementedError()

    def count_check(self, table_name):
        NotImplementedError()

    def hist_count_check(self, table_name):
        NotImplementedError()




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ovotestkit", # Replace with your own username
    version="1.0.0",
    author="OVO BD Enginners, Zaky Rahim",
    author_email="bd-engineers@ovo.id, zaky.rahim@ovo.id",
    description="Testing Kit for OVO Big Data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'psycopg2-binary',
        'pandas',
        'pyspark',
        'bash',
        'sqlalchemy'
    ],
    python_requires='>=3.6',
)